from typing import Any, Tuple
import _pickle as cPickle

import numpy as np

from landsklim.lisdqs.lisdqs_analysis import LISDQSAnalysis
from landsklim.lisdqs.lisdqs_project import LISDQSProject
from landsklim.lisdqs.polygons_definition import PolygonsDefinition


class PearsonRResultDummyClass:
    """
    In Landsklim 0.5.1, LISDQSAnalysis.self._pearson_correlation[...][...] can be pickled as PearsonRResult.
    Following version of Scipy, PearsonRResult is sometimes a simple tuple.
    From Landsklim 0.6.0, LISDQSAnalysis.self._pearson_correlation[...][...] is always serialized as tuple
    PearsonRResultDummyClass is there to make the bridge if a user loads a project with PearsonRResult
     but the scipy version installed doesn't have an implementation of PearsonRResult
    """

    def __new__(cls, *args, **kwargs):
        obj = super().__new__(cls)
        obj.tuple = args
        return obj

    def __getitem__(self, index):
        return self.tuple[index]

    def to_json(self) -> Tuple[float, float]:
        return self.tuple

    def __repr__(self):
        return "PearsonRResultDummyClass{0}".format(self.tuple)

class LandsklimUnpickler(cPickle.Unpickler):

    @staticmethod
    def postprocess(landsklim_project: LISDQSProject):
        for analysis in landsklim_project.get_analysis():  # type: LISDQSAnalysis
            if not analysis.is_local():
                stations_count = analysis.get_stations_count()
                analysis._polygons = PolygonsDefinition(None, np.linspace(0, stations_count - 1, stations_count, dtype=int).reshape(1, -1), [[]], None)

    def find_class(self, module_name: str, global_name: str) -> Any:
        if global_name == "LISDQSRectangle":
            global_name = "LandsklimRectangle"
        if global_name == "PearsonRResult":
            module_name = "landsklim.serialization.landsklim_unpickler"
            global_name = "PearsonRResultDummyClass"
        return super(LandsklimUnpickler, self).find_class(module_name, global_name)
